library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.generic_type_pkg.all;

entity tb_dptc_basic is
end entity;

architecture Behavioral_basic of tb_dptc_basic is
  constant Nv : integer := 2;
  constant Nh : integer := 2;
  constant N  : integer := 4;
  constant CUSTOM_WIDTH : integer := 16;

  signal clk           : std_logic := '0';
  signal reset_n       : std_logic := '0';
  signal enable        : std_logic := '0';
  signal out_valid     : std_logic;

  signal x_matrix      : my_custom_matrix(0 to Nv-1, 0 to N-1);
  signal y_matrix      : my_custom_matrix(0 to Nh-1, 0 to N-1);
  signal result_matrix : dot_matrix_type(0 to Nv-1, 0 to Nh-1);

  constant clk_period : time := 10 ns;

begin

  -- Instantiate the DUT
  uut: entity work.dptc
    generic map (
      Nv => Nv,
      Nh => Nh,
      N  => N,
      CUSTOM_WIDTH => CUSTOM_WIDTH
    )
    port map (
      clk           => clk,
      reset_n       => reset_n,
      enable        => enable,
      x_matrix      => x_matrix,
      y_matrix      => y_matrix,
      out_valid     => out_valid,
      result_matrix => result_matrix
    );

  -- Clock generation
  clk_process : process
  begin
    clk <= '0';
    wait for clk_period/2;
    clk <= '1';
    wait for clk_period/2;
  end process;

  -- Stimulus process
  stim_proc: process
  begin
    reset_n <= '0';
    enable <= '0';
    wait for 20 ns;

    reset_n <= '1';
    wait for clk_period;

    -- Assign test values using to_signed
    x_matrix(0,0) <= to_signed(1, CUSTOM_WIDTH);
    x_matrix(0,1) <= to_signed(2, CUSTOM_WIDTH);
    x_matrix(0,2) <= to_signed(3, CUSTOM_WIDTH);
    x_matrix(0,3) <= to_signed(4, CUSTOM_WIDTH);

    x_matrix(1,0) <= to_signed(5, CUSTOM_WIDTH);
    x_matrix(1,1) <= to_signed(6, CUSTOM_WIDTH);
    x_matrix(1,2) <= to_signed(7, CUSTOM_WIDTH);
    x_matrix(1,3) <= to_signed(8, CUSTOM_WIDTH);

    y_matrix(0,0) <= to_signed(1, CUSTOM_WIDTH);
    y_matrix(0,1) <= to_signed(0, CUSTOM_WIDTH);
    y_matrix(0,2) <= to_signed(1, CUSTOM_WIDTH);
    y_matrix(0,3) <= to_signed(0, CUSTOM_WIDTH);

    y_matrix(1,0) <= to_signed(0, CUSTOM_WIDTH);
    y_matrix(1,1) <= to_signed(1, CUSTOM_WIDTH);
    y_matrix(1,2) <= to_signed(0, CUSTOM_WIDTH);
    y_matrix(1,3) <= to_signed(1, CUSTOM_WIDTH);

    wait for clk_period;
    enable <= '1';

    -- Wait for valid signal
    wait until rising_edge(clk) and out_valid = '1';
    enable <= '0';
    wait for 50 ns;

    -- Report results
    report "Result Matrix (After Multiplication): ";
    report "Row 0, Col 0: " & integer'image(to_integer(result_matrix(0,0)));
    report "Row 0, Col 1: " & integer'image(to_integer(result_matrix(0,1)));
    report "Row 1, Col 0: " & integer'image(to_integer(result_matrix(1,0)));
    report "Row 1, Col 1: " & integer'image(to_integer(result_matrix(1,1)));

    wait;
  end process;

end architecture;
