library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.generic_type_pkg.all;

entity ddot_unit is
  generic (
    N : integer := 2;
    CUSTOM_WIDTH : integer := 16
    
  );
  port (
    clk     : in std_logic;
    reset_n : in std_logic;
    enable  : in std_logic;
    x_vec   : in my_custom_vector(0 to N-1);
    y_vec   : in my_custom_vector(0 to N-1);
    out_valid : out std_logic;
    dot_out   : out dot_product_type -- safe sum width
  );
end entity;

architecture Behavioral of ddot_unit is

  begin
  process(clk, reset_n)
    constant PROD_WIDTH : integer := 2 * CUSTOM_WIDTH;
    constant SUM_WIDTH  : integer := PROD_WIDTH + 5;  -- enough bits to sum N products

    type product_array_type is array(0 to N-1) of product_type;
    variable partial_products : product_array_type;
    variable temp_sum         : sum_type;
    variable temp_valid       : std_logic;


  begin
    if reset_n = '0' then
      dot_out    <= (others => '0');
      out_valid  <= '0';
      temp_sum   := (others => '0');
      temp_valid := '0';

    elsif rising_edge(clk) then
      if enable = '1' then
        -- Multiply vector elements
        for i in 0 to N-1 loop
        -- Multiply then resize result
        partial_products(i) := resize(x_vec(i) * y_vec(i), PROD_WIDTH);
      end loop;


        -- Sum the partial products
        temp_sum := (others => '0');
        for i in 0 to N-1 loop
          temp_sum := resize(temp_sum, SUM_WIDTH) + resize(partial_products(i), SUM_WIDTH);
        end loop;

        dot_out   <= temp_sum;
        out_valid <= '1';

      else
        out_valid <= '0';
      end if;
    end if;
  end process;
end architecture;

