library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.generic_type_pkg.all;  -- must match your custom vector types

entity tb_ddot_unit is
end tb_ddot_unit;

architecture behavior of tb_ddot_unit is

  constant N : integer := 4;
  constant CUSTOM_WIDTH : integer := 16;
  constant DOT_WIDTH : integer := 2 * CUSTOM_WIDTH + 5;

  signal clk       : std_logic := '0';
  signal reset_n   : std_logic := '0';
  signal enable    : std_logic := '0';
  signal x_vec     : my_custom_vector(0 to N-1);
  signal y_vec     : my_custom_vector(0 to N-1);
  signal out_valid : std_logic;
  signal dot_out   : signed(DOT_WIDTH-1 downto 0);

  component ddot_unit is
    generic (
      N : integer := 2;
      CUSTOM_WIDTH : integer := 16
    );
    port (
      clk     : in std_logic;
      reset_n : in std_logic;
      enable  : in std_logic;
      x_vec   : in my_custom_vector(0 to N-1);
      y_vec   : in my_custom_vector(0 to N-1);
      out_valid : out std_logic;
      dot_out   : out signed(DOT_WIDTH-1 downto 0)
    );
  end component;

begin

  -- Instantiate the Unit Under Test (UUT)
  uut: ddot_unit
    generic map (
      N => N,
      CUSTOM_WIDTH => CUSTOM_WIDTH
    )
    port map (
      clk => clk,
      reset_n => reset_n,
      enable => enable,
      x_vec => x_vec,
      y_vec => y_vec,
      out_valid => out_valid,
      dot_out => dot_out
    );

  -- Clock generation
  clk_process : process
  begin
    while true loop
      clk <= '0';
      wait for 5 ns;
      clk <= '1';
      wait for 5 ns;
    end loop;
  end process;

  -- Stimulus process
  stim_proc: process
    variable expected_result : integer;
  begin
    -- Initialize
    reset_n <= '0';
    enable  <= '0';
    wait for 20 ns;
    reset_n <= '1';

    -- Apply first input
    wait for 10 ns;
    enable <= '1';

    -- Assign test vectors
    for i in 0 to N-1 loop
      x_vec(i) <= to_signed(i+1, CUSTOM_WIDTH);  -- (1, 2, 3, 4)
      y_vec(i) <= to_signed(i+1, CUSTOM_WIDTH);  -- (1, 2, 3, 4)
    end loop;
    expected_result := 1*1 + 2*2 + 3*3 + 4*4;  -- 30

    wait for 10 ns;
    enable <= '0';

    wait for 10 ns;

    report "Expected dot product = " & integer'image(expected_result);
    report "Actual dot product   = " & integer'image(to_integer(dot_out));
    assert to_integer(dot_out) = expected_result
      report "Mismatch in dot product!"
      severity error;

    wait;

  end process;

end behavior;
