library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

package generic_type_pkg is
    constant MAX_N : integer := 32;
    constant CUSTOM_WIDTH : integer := 16;
    subtype my_custom_element is signed(CUSTOM_WIDTH - 1 downto 0);  -- 16-bit signed integer
    type my_custom_vector is array(natural range <>) of my_custom_element;
    type my_custom_matrix is array(natural range <>, natural range <>) of my_custom_element;
    subtype dot_product_type is signed(2*CUSTOM_WIDTH + 5 - 1 downto 0);
    type dot_matrix_type is array(natural range <>, natural range <>) of dot_product_type;
    subtype sum_type is signed(2*CUSTOM_WIDTH + 5 - 1 downto 0);
    subtype product_type is signed(2*CUSTOM_WIDTH - 1 downto 0);


    -- Type-dependent constants
    constant ZERO_GEN : my_custom_element := (others => '0');
    constant ONE_GEN  : my_custom_element := to_signed(1, CUSTOM_WIDTH);
end package;

package body generic_type_pkg is
end package body;
