library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use std.textio.all;
use ieee.std_logic_textio.all;
use work.generic_type_pkg.all;

entity tile_array_tb is
end tile_array_tb;

architecture Behavioral of tile_array_tb is
  constant Np : integer := 4;
  constant Nm : integer := 4;
  constant Nq : integer := 4;
  constant Nt : integer := 2;
  constant Nd : integer := 2;
  constant Nh : integer := 2;
  constant N  : integer := 2;
  constant clk_period : time := 10 ns;

  signal clk          : std_logic := '0';
  signal reset_n      : std_logic := '0';
  signal enable       : std_logic := '0';
  signal m1           : my_custom_matrix(0 to Np-1, 0 to Nm-1);
  signal m2           : my_custom_matrix(0 to Nq-1, 0 to Nm-1);
  signal result_valid : std_logic;
  signal result       : dot_matrix_type(0 to Np-1, 0 to Nq-1);

begin

  -- Clock generation
  clk_process : process
  begin
    while true loop
      clk <= '0';
      wait for clk_period / 2;
      clk <= '1';
      wait for clk_period / 2;
    end loop;
  end process;

  -- Unit Under Test
  uut: entity work.tile_array
    generic map (
      Np => Np, Nm => Nm, Nq => Nq, Nt => Nt,
      Nd => Nd, Nh => Nh, N => N
    )
    port map (
      clk          => clk,
      reset_n      => reset_n,
      enable       => enable,
      m1           => m1,
      m2           => m2,
      result_valid => result_valid,
      result       => result
    );

  -- Stimulus Process
  stim_proc: process
    variable i, j : integer;
    variable L : line;
    file out_file : text open write_mode is "tile_array_out.txt";
    file m1_file  : text open write_mode is "m1.txt";
    file m2_file  : text open write_mode is "m2.txt";
  begin
    reset_n <= '0';
    wait for 2 * clk_period;
    reset_n <= '1';
    wait for clk_period;

    -- Initialize m1 matrix
    for i in 0 to Np-1 loop
      for j in 0 to Nm-1 loop
        m1(i, j) <= to_signed(i * Nm + j + 1, CUSTOM_WIDTH);
      end loop;
    end loop;

    wait for clk_period;

    -- Dump m1 matrix to file
    for i in 0 to Np-1 loop
      L := null;
      for j in 0 to Nm-1 loop
        write(L, integer'image(to_integer(m1(i,j))));
        write(L, string'(" "));
      end loop;
      writeline(m1_file, L);
    end loop;

    -- Initialize m2 matrix
    for i in 0 to Nq-1 loop
      for j in 0 to Nm-1 loop
        m2(i, j) <= to_signed((i + 1) * 5 + j + 1, CUSTOM_WIDTH);
      end loop;
    end loop;

    wait for clk_period;

    -- Dump m2 matrix to file
    for i in 0 to Nq-1 loop
      L := null;
      for j in 0 to Nm-1 loop
        write(L, integer'image(to_integer(m2(i,j))));
        write(L, string'(" "));
      end loop;
      writeline(m2_file, L);
    end loop;

    wait for clk_period * 2;
    enable <= '1';

    -- Wait for result_valid
    wait until rising_edge(result_valid);
    enable <= '0';
    wait for clk_period;

    -- Print result to file and console
    write(L, string'("===== FINAL RESULT ===== at time " & time'image(now)));
    writeline(out_file, L);

    for i in 0 to Np-1 loop
      L := null;
      for j in 0 to Nq-1 loop
        write(L, integer'image(to_integer(result(i,j))));
        write(L, string'(" "));
      end loop;
      writeline(out_file, L);
    end loop;

    report "Matrix multiplication result is valid.";
    for i in 0 to Np-1 loop
      for j in 0 to Nq-1 loop
        report "Result(" & integer'image(i) & "," & integer'image(j) & ") = " &
               integer'image(to_integer(result(i, j)));
      end loop;
    end loop;

    wait;
  end process;

end Behavioral;
